#pragma once

#include <vector>
#include <fstream>
#include <iostream>
#include <filesystem>

#define VULKAN_HPP_NO_STRUCT_CONSTRUCTORS
#define VULKAN_HPP_DISPATCH_LOADER_DYNAMIC 1
#include <vulkan/vulkan.hpp>

inline std::uint32_t find_memory_type(const vk::PhysicalDevice& device, const std::uint32_t typeFilter, const vk::MemoryPropertyFlags properties) {
	const auto result = device.getMemoryProperties();

	for (uint32_t i = 0; i < result.memoryTypeCount; i++) {
		if ((typeFilter & (1 << i)) && (result.memoryTypes[i].propertyFlags & properties) == properties) {
			return i;
		}
	}

	return -1;
}

inline VKAPI_ATTR VkBool32 VKAPI_CALL debug_callback(const VkDebugUtilsMessageSeverityFlagBitsEXT messageSeverity,
													 const VkDebugUtilsMessageTypeFlagsEXT messageType,
													 const VkDebugUtilsMessengerCallbackDataEXT* pCallbackData,
													 void* pUserData) {

	if (messageSeverity & VK_DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT) {
		std::cerr << "ERROR: " << (pCallbackData->pMessage) << std::endl;
	} else if (messageSeverity & VK_DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT) {
		std::cout << "WARN: " << (pCallbackData->pMessage) << std::endl;
	} else if ((messageSeverity & VK_DEBUG_UTILS_MESSAGE_SEVERITY_VERBOSE_BIT_EXT) ||
			   (messageSeverity & VK_DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT)) {
		std::cout << "INFO: " << (pCallbackData->pMessage) << std::endl;
	}

	return VK_FALSE;

}

inline std::vector<std::uint32_t> read_file(const std::filesystem::path& path) {
	std::fstream file(path, std::ios::in | std::ios::ate | std::ios::binary);
	std::uint64_t size = file.tellg();
	std::vector<std::uint32_t> buffer(size / 4);

	file.seekg(0);
	file.read(reinterpret_cast<char*>(buffer.data()), size);
	file.close();

	return buffer;
}