#version 460 core
#extension GL_EXT_nonuniform_qualifier : require

struct Material {
	uint textures;
};

// needs to by synced in cpp
#define COUNT 256

layout(std430, set = 0, binding = 1) readonly buffer MaterialBuffer {
	Material materials[COUNT];
};

layout(set = 0, binding = 2) uniform sampler2D material_textures[COUNT * 9u];

layout(location = 0) flat in uint fsMaterialIndex;

layout(location = 0) out vec4 outColor;

#define TRY_TEST 0

#if TRY_TEST == 0
// this one stops it from creating the pipeline
// or it will become really slow 
// tested with COUNT == 32 in real application, (not minimal reproducible), 
// where it took ~5sec to create 1 pipeline
vec4 test(Material materials[COUNT], sampler2D textures[COUNT * 9u], uint material_index) {
	const uint index = 3u;

	if (bool(materials[fsMaterialIndex].textures & (1u << index))) {
		return texture(textures[fsMaterialIndex * 9u + index], vec2(0.5f));
	} else {
		return vec4(0.0f);
	}
}
#elif TRY_TEST == 1
// this one just works
vec4 test(Material materials[COUNT], uint material_index) {
	const uint index = 3u;

	if (bool(materials[fsMaterialIndex].textures & (1u << index))) {
		return texture(material_textures[fsMaterialIndex * 9u + index], vec2(0.5f));
	} else {
		return vec4(0.0f);
	}
}
#elif TRY_TEST == 2
// this one also does not work
vec4 test(sampler2D textures[COUNT * 9u], uint material_index) {
	const uint index = 3u;

	if (bool(materials[fsMaterialIndex].textures & (1u << index))) {
		return texture(textures[fsMaterialIndex * 9u + index], vec2(0.5f));
	} else {
		return vec4(0.0f);
	}
}
#endif

void main() {
#if TRY_TEST == 0
	outColor = test(materials, material_textures, fsMaterialIndex);
#elif TRY_TEST == 1
	outColor = test(materials, fsMaterialIndex);
#elif TRY_TEST == 2
	outColor = test(material_textures, fsMaterialIndex);
#endif
}